% (Ktrain, Ktest) =
%     GetKernelRepresentation(Atrain, Atest, subsample_proportion, tau)
%
% Kernelizes the raw data matrices in Atrain and Atest, returning
% kernel train and test matrices for use in classification schemes.
% The kernel function used is
%
%   K(a1, a2) = exp(-norm(a1 - a2)^2 / (2 * tau^2)).
%
% The matrices Ktrain and Ktest are constructed as follows. The matrix
% Atrain is of size Ntrain-by-d, Atest is of size Ntest-by-d, where
% Ntrain/Ntest are the training and test set sizes and d is the
% dimension of the raw data.
%
% Chooses a random fraction subsample_proportion (yielding a total Nk
% indices) of the training data to use in constructing the kernel
% matrices, call the random indices chosen
%
%   {i(1), i(2), ..., i(Nk)}
%
% Then Ktrain is an Ntrain-by-Nk matrix whose entries are
%
%  Ktrain[i, j] = K(Atrain[i, :], Atrain[i(j), :])
%
% while Ktest is an Ntest-by-Nk matrix whose entries are
%
%  Ktest[i, j] = K(Atest[i, :], Atrain[i(j), :]).
%
%
function [Ktrain, Ktest] = ...
    GetKernelRepresentation(Atrain, Atest, ...
                            subsample_proportion, tau)
if (nargin < 3)
  subsample_proportion = .1;
end
if (nargin < 4)
  tau = 1;
end
if (subsample_proportion <= 0 || subsample_proportion > 1)
  error(['Proportion for subsampling training data must be in (0, 1], was ' ...
         '%f'], subsample_proportion);
end
[Ntrain, dd] = size(Atrain);
[Ntest, dd] = size(Atest);
% Find number of random entries to kernelize and their indices
num_to_kernelize = ceil(subsample_proportion * Ntrain);
kernelized_inds = randperm(Ntrain);
kernelized_inds = kernelized_inds(1:num_to_kernelize);
% Compute the distance matrices for train
squared_trains = sum(Atrain.^2, 2);
inner_product_trains = Atrain * Atrain(kernelized_inds, :)';
squared_train_distances = ...
    (repmat(squared_trains, 1, num_to_kernelize) ...
     + repmat(squared_trains(kernelized_inds)', Ntrain, 1) ...
     - 2 * inner_product_trains);
% Compute the distance matrices for test
squared_tests = sum(Atest.^2, 2);
inner_product_train_test = Atest * Atrain(kernelized_inds, :)';
squared_test_distances = ...
    (repmat(squared_tests, 1, num_to_kernelize) ...
     + repmat(squared_trains(kernelized_inds)', Ntest, 1) ...
     - 2 * inner_product_train_test);
% Get the kernels themselves
Ktrain = exp(-squared_train_distances / (2 * tau^2));
Ktest = exp(-squared_test_distances / (2 * tau^2));

